#Josef Laimer, University of Salzburg, 2010-2016
import chimera, os, sys, subprocess
from chimera.baseDialog import ModelessDialog
from chimera import dialogs
from chimera.resCode import *
import StringIO 
import urllib2, urllib, json
import requests

#ASA_SERVER_URL = "http://chimera2016.projekte.fh-hagenberg.at/biodapps/rest/%s"
ASA_SERVER_URL = "https://biwww.che.sbg.ac.at/biodapps/pl/rest/%s"

def startGUI ():
    import BioDApps_AccSurfSelectionWeb
    import BioDApps_AccSurfSelectionWeb.gui
    from chimera import dialogs
    dialogs.display(BioDApps_AccSurfSelectionWeb.gui.AccSurfDialog().name)    

class ASAHandler ():
    def __init__(self):
        self.accessiblities = {} #oslId->probesize->residue->value
        self.jobId = None
        self.uploadedFile = {}
        self.url = ASA_SERVER_URL
    
    def __createJob (self):
        if self.jobId == None:
            urlJob = self.url % "getJobId"
            data = {"service":"ASA"}
            data = json.dumps(data)
            #send request
            req = urllib2.Request(urlJob, data, {'Content-Type': 'application/json'})
            response = urllib2.urlopen(req)
            result = json.load(response)
            self.jobId = result["jobId"]
    
    def __uploadFile (self, molosl, molecule):
        if molosl in self.uploadedFile:
            return self.uploadedFile[molosl][0]
        
        strfi = StringIO.StringIO()
        chimera.pdbWrite([molecule], molecule.openState.xform, strfi)
        strfi.seek(0,0) #zurueck auf erste zeile (zeiger war auf der letzten zeile -> deshalb gabs keine ausgabe)
    
        response = requests.post(self.url % "upload", files={'upload': strfi})
        self.uploadedFile[molosl] = eval(response.text)["filenames"]
    
        #- move file to job folder (server)
        data = {"service" : "Mollib", 
                "jobId" : self.jobId, 
                "param" : [{'pdbFile': self.uploadedFile[molosl][0], 'orig_pdbFile' : molosl, 'command' : 'upload'}]};
    
        data = json.dumps(data)
        req = urllib2.Request(self.url % "runService", data, {'Content-Type': 'application/json'})
        response = urllib2.urlopen(req)
        result = json.load(response)
        
        return self.uploadedFile[molosl][0]
        
    def getValues (self, molOSL, probesize):
        self.__createJob()
        if self.accessiblities.has_key(molOSL) and self.accessiblities[molOSL].has_key(probesize):
            return self.accessiblities[molOSL][probesize]
        aList = []
        
        mol = chimera.selection.OSLSelection(molOSL)
        mol = mol.molecules()[0]
 
        serverFilename = self.__uploadFile(molOSL, mol)
        
        resAsa = self.__calcResidueASA(self.__runASA(serverFilename, probesize))
        
        if not self.accessiblities.has_key(molOSL):
            self.accessiblities[molOSL] = {}
        
        self.accessiblities[molOSL][probesize] = resAsa
        return resAsa

    def __runASA(self, serverfile, probesize):
        data = {"service": "ASA",
        "jobId": self.jobId,
        "param": [{
        'pdbFile': serverfile, 
        'bu' : False, 
        'probesize': probesize}]}    #TODO bu?
    
        data = json.dumps(data)
        req = urllib2.Request(self.url % "runService", data, {'Content-Type': 'application/json'})
        response = urllib2.urlopen(req)
        result = json.load(response)
        return result
    
    def __calcResidueASA (self, asavalues):
        asavalues = asavalues["result"]
        asa = asavalues["asa"]
        rasa = asavalues["rasa"]
        asavalues["maxAsa"] = max(asa)
        asavalues["maxRAsa"] = max(rasa)
        return asavalues
    
        