# GUI #
from Tkinter import *  # @UnusedWildImport
from Tkconstants import *  # @UnusedWildImport
import Tkconstants as tkc

# Normal #
import multiListbox as mlb
from chimera import dialogs
from chimera.baseDialog import ModelessDialog
import joblist  
import BioDApps_Pirates.gui
import BioDApps_Maestro.gui
#import BioDApps_Maestro_Score.gui


# BioDAppsExtension #
#######################################################################################
class BioDAppsDialog(ModelessDialog):
    """
    BioDApps 
    inherited from ModelessDialog  
    Fills GUI.
    GUI has a Table(MultiListBox) (jobID,Service,Date)
    In the Table are saved all sended jobs (newer than 7 days)
    This GUI is to reload old or running jobs 
    it works with a joblist.txt where the jobs a persisted 
    if a new job is sent The GUI will be updated because its an observer of this joblist     
    """
    
    title="BioDApps"
    name="BioDApps"
    buttons=('Send','Close')   
    
    # INIT #
    def __init__(self, **kw):
        """
        Constructor
        contains the joblist Singelton
        and register itself as observer to the joblist
        """        
        self.frame1 = None
        self.mlb = None
        self.jobList = joblist.JobList.getInstance()
        ModelessDialog.__init__(self, **kw)
        
        self.jobList.register(self)     
        
    # OBSERVE # 
    def update(self, *args, **kwargs):
        """
        function of the observer pattern
        called when a new job has been added to the joblist
        """
        self.loadJobs()
    
    #  BUTTONS #
    def Send(self):
        """
        Send 
        imports the saved session to th last selected job opens it and 
        loads the resultgui iwth the data from the session and the result
        """    
        
        if (self.mlb.size()): 
            if len(self.mlb.curselection()) != 0:  
                jobId = self.mlb.getLastSelection()
                service = self.mlb.getLastSelectionSec()
                if(self.jobList.importSession(jobId)):       
                    if (service == "Pirates"):
                        BioDApps_Pirates.gui.getResultById(jobId)
                    else:
                        BioDApps_Maestro.gui.getResultById(jobId)
#                    elif (service == "MaestroS"):
#                        BioDApps_Maestro_Score.gui.getResultById(jobId)
           
    def Close(self):   
        """
        Closes the GUI and deregister the observer
        """    
        self.jobList.deregister(self) 
        self.mlb.destroy()
        ModelessDialog.Close(self)    
        self.destroy()    
          
    # CODE #   
    def loadJobs(self):
        """
        load the all old/running jobs of the joblist into the Multilistbox of the GUI
        to represent the data
        """
        if (len(self.mlb.lists) > 0):
            self.mlb.deleteAll()
        entryList = self.jobList.loadFile();
        for entry in entryList:
            if len(entry) > 2:
                self.setListData(entry[0], entry[1], entry[2])
    
    def setListData(self,jobID,service,date): 
        """
        @param jobID: unique ID of the job to reload it/identify the job on the server
        @param service: which service was used was sent (Maestro, MaestroS or Pirates)
        @param date: current date 
        sets a single entry and justifys multilistbox
        """  
        self.mlb.insert(tkc.END, (jobID, service ,date))
        self.mlb.justify()
                  
    # GUI # 
    def fillInUI(self, parent):
        """
        Has the representation Container of the GUI a Multilistbox      
        GUI has a Table(MultiListBox) (jobID,Service,Date)
        In the Table are saved all sended jobs (newer than 7 days)
        to reload a old job         
        """
          
        self.parent = parent
        self.frame1=Frame(parent, bg="#d3ebf7")
        self.frame1.pack(padx=0,pady=0,expand=YES,fill=BOTH)
        
        labelframe1 = LabelFrame(self.frame1, text="JobList", bg="#d3ebf7")
        labelframe1.pack(fill=BOTH, expand=YES, padx=0, pady=5)
        
        self.mlb = mlb.MultiListbox(labelframe1, True,  (('JobID',  50, StringType, tkc.LEFT), 
                                                        ('Service', 15, StringType, tkc.LEFT),
                                                        ('Date',    20, StringType, tkc.LEFT)))
     
        self.mlb.pack(side=LEFT, fill=BOTH, expand=YES)
        self.mlb.justify()       
        if (self.jobList.getLength() > 0):     
            self.jobList.deleteOldData()
        self.loadJobs()  
  
dialogs.register(BioDAppsDialog.name, BioDAppsDialog)  