import os
from chimera import preferences
import SimpleSession
from datetime import datetime
from chimera import runCommand
import tkMessageBox

class JobList():     
    ########## SINGLETON ##########
    __instance = None
    
    @classmethod
    def getInstance(cls):
        """
        necessary for the singleton pattern
        makes a new instance if there isnt an instance otherwise it retrns the instance
        """
        if cls.__instance == None:
            cls.__instance = cls()
        return cls.__instance
    
    def __init__(self):
        """
        necessary for the singleton pattern
        normal Contructor"
        """
        self.lastId = 0
        self.joblist = []
        self.cPath = os.path.join(os.path.dirname(preferences.preferences._filename), "biodappsJobs.csv")
        self.sPath = os.path.join(os.path.dirname(preferences.preferences._filename), "sessions")
        self.observers = []
                
    def reset(self):
        """
        necessary for the singleton pattern
        resets the id of the singleton
        """
        self.lastId = 0
        
    def getNextId (self):        
        """
        necessary for the singleton pattern
        returns the next id of the singleton
        """
        self.lastId += 1
        return self.lastId
    
    def getId (self):
        """
        necessary for the singleton pattern
        returns the last id of the singleton
        """
        return self.lastId
    ########## SINGLETON ##########    
  
    ########## OBSERVABLE ##########
    def register(self, observer):
        """
        necessary for the observer pattern
        register given observer to the observerlist
        
        @param: observer: observer that should be registered
        """
        if not observer in self.observers:
            self.observers.append(observer)
    
    def deregister(self, observer):
        """
        necessary for the observer pattern
        deregister given observer from the observerlist
        
        @param: observer: observer that should be deregistered
        """
        if observer in self.observers:
            self.observers.remove(observer)
 
    def updateObservers(self, *args, **kwargs):
        """
        necessary for the observer pattern
        updates all the observers
        """
        for observer in self.observers:
            observer.update(*args, **kwargs)
    ########## OBSERVABLE ##########


    ########## SAVE/LOAD ##########  
    def saveFile(self,jobID,service): 
        """
        saves a job (jobId, service, time) to the joblist.txt
        to persist the data and make it possible the reload jobs later
        and updates the observer
        
        @param jobID: unique ID of the job to reload it/identify the job on the server
        @param service: which service was used was sent (Maestro, MaestroS or Pirates)
        """
        self.exportSession(jobID)
 
        now = datetime.now()
        date = str(now).split(".")[0]
               
        if (os.path.exists(self.cPath)):
            fJobList = file(self.cPath, 'a')
        else:
            fJobList = file(self.cPath, 'w')
        fJobList.write(str(jobID + ";" + service + ";" + date +"\n"))
        fJobList.close()
        self.updateObservers(jobID)
        
    def _saveFile(self,jobID,service,date):  
        """
        saves a job (jobId, service, time) to the joblist.txt
        to persist the data and make it possible the reload jobs later
        and does NOT update the observer
        
        @param jobID: unique ID of the job to reload it/identify the job on the server
        @param service: which service was used was sent (Maestro, MaestroS or Pirates)
        @param date: current date 
        """         
        if (os.path.exists(self.cPath)):
            fJobList = file(self.cPath, 'a')
        else:
            fJobList = file(self.cPath, 'w')
        fJobList.write(str(jobID + ";" + service + ";" + date +"\n"))
        fJobList.close() 
             
    def loadFile(self):
        """
        loads all jobs (jobId, service, time) of the joblist.txt into the return variable self.joblist
        @return: self.joblist: joblist with jobId, service and time
        @type: list<str>
        """
        self.joblist = []
        if (os.path.exists(self.cPath)):
            rJobList = open(self.cPath, 'r')
            for line in rJobList:
                line.rstrip()
                entry = line.split(';')
                self.joblist.append(entry)    
            rJobList.close()
        return self.joblist;
    ########## SAVE/LOAD ##########          
    
    ########## SESSIONS ##########
    def exportSession(self,jobID):  
        """
        exports a session to the filessystem of the computer to persist it to reload it later
        name of the file is the unique jobId
        
        @param jobID: unique ID of the job to reload it/identify the job on the server
        """     
        if (not os.path.exists(self.sPath)):
            os.mkdir(self.sPath)
        nsPath = self.__getFileName(jobID)    
        SimpleSession.saveSession(nsPath)

    def importSession (self,jobID):
        """
        imports a session into Chimera by execution of the file
        name of the file is the unique jobId
        
        @param jobID: unique ID of the job to reload it/identify the job on the server
        """

        result = tkMessageBox.askquestion("New Session", "There are models open. Close them before restoring the session?", icon='warning')
        print("RESULT: ", result)
        if result == "yes":
            print("RESULT: ", result)
            runCommand("close session")
            nsPath = self.__getFileName(jobID)
            execfile(nsPath, globals(), globals())
            return True
        return False
        
        
    def __getFileName(self,jobID):
        """   
        subfunction of import/save Session    
        returns the name of the file of the right session (specified by the jobID)
        
        @param jobID: unique ID of the job to reload it/identify the job on the server
        @return nsPath: name of the wanted sessionfile (matched to the ID)
        @type nsPath: str
        """
        sessionName = str(jobID + ".py")  
        nsPath = os.path.join(self.sPath, sessionName)
        return nsPath
    ########## SESSIONS ##########

    def deleteOldData(self):
        """
        proof s if the entrys in the joblist.txt are older than 7 Days
        if yes than this jobentry will be deleted otherwise not
        (delets old joblist and refills with right data) 
        """
        now = datetime.now()
        ndate = str(now).split(".")[0];
        entryList = self.loadFile();
        saveList = entryList
        for entry in entryList:
            if len(entry) > 2:
                odate = entry[2]
                
                ndate = str(ndate).split(" ")[0]
                odate = str(odate).split(" ")[0]
             
                date_format = "%Y-%m-%d"
                ndate = datetime.strptime(ndate, date_format)
                odate = datetime.strptime(odate, date_format)
                diff = ndate - odate
                
                if (diff.days > 7):
                    saveList.remove(entry)
                    
        self.clearFile()    
        
        for entry in saveList:
            if len(entry) > 2:
                self._saveFile(entry[0], entry[1], entry[2])    
            
    def clearFile(self):  
        """
        clears the joblistFile
        """
        open(self.cPath, 'w').close()
      
    def getLength(self):
        """
        returns the number of lines
        @return count: number of lines in the File/joblist
        @type count: int
        """
        count = 0
        if (os.path.exists(self.cPath)):
            rJobList = open(self.cPath, 'r')
            for line in rJobList:  
                line
                count += 1  
            rJobList.close()
        return count  
    
    
    
    
    