import string
import chimera
import sys
import time
from threading import Thread
import re # RegEx for validation

from tkFileDialog import askopenfilename
from Tkinter import *
from chimera.baseDialog import ModelessDialog
import os
import Pmw
from Pmw import ComboBox  # @UnresolvedImport
from FlattenIcosahedron.flatten import length
from chimera.selection import REPLACE



def validateMaestro(MaestroGUI):
    '''
    validates input of MaestroGUI before data will be extracted
    when not valid: prints Error-Message to idle and returns a tuple containing FALSE and error msg 
    when get's through all tests without detecting an error return TRUE at the end
    validates if:
        if evaluate specific mutations:
            -> are there any mutations defined (if none Job won't be sent)
            -> are mutations vaild
                do mutations fit to pattern (RegEx)
                do they exist
                don't they mutate to themselves
            -> is ph-value valid
        if sensitivity profile: not validations necessary
            
    @param MaestroGUI: gets whole GUI where data was inserted by user, so it can extract the values
    @return: infoPair: First Value says if validation is valid or not, Second Value says the errormessage if first is false
    @type param: pair (boolean|str)
            
    '''
    
    infoPair = (True,'')    #errorMsg = '' valid = True
    
    if(MaestroGUI.choice.get() == 2):   #if choice is on evaluate specific mutationes
        
        #are there mutationes defined?
        if ( MaestroGUI.text1.get('1.0', 'end-1c')==''):
            errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, mutationlist is empty!\n Please add your required mutations in the 'List of Mutations'!\n"
            print errorMsg
            infoPair=(False,errorMsg)
            return infoPair
        
        #are mutations valid?
        #do entries in text box fit to pattern?
        # moved to  checkResiduesTextbox: p = re.compile('^(([A-Z])[0-9]+\.[A-Z])\{([A-Z]([A-Z, ]*[A-Z]+)?)\}$')    #A1.A{T,L} A90.C{SMALL} ...
        
        newInfo = checkResiduesTextbox(MaestroGUI, MaestroGUI.text1, MaestroGUI.Mutate_from)   #selectableResidues = MaestroGUI.Mutate_from
        infoPair = (infoPair[0] and newInfo[0],infoPair[1] + newInfo[1])
        #is ph between 5.5 and 8.5 (max. 2 decimal places)?
        ph = MaestroGUI.ph_value.get()
        newInfo = checkPhValue(ph)
        infoPair = (infoPair[0] and newInfo[0],infoPair[1] + newInfo[1])
    
    if(MaestroGUI.choice.get() == 3):   #if choice is on Scan for (de)stabilizing mutations
        #do entries in text box fit to pattern?
        if ( MaestroGUI.text1.get('1.0', 'end-1c')!=''):
            # delete this line p = re.compile('^(([A-Z])[0-9]+\.[A-Z])(\{X\})?$')    #A1.A A90.C{X} ...
            newInfo = checkResiduesTextbox(MaestroGUI,MaestroGUI.text1, MaestroGUI.Mutate_from)
            infoPair = (infoPair[0] and newInfo[0],infoPair[1] + newInfo[1])
            
        #is ph between 5.5 and 8.5 (max. 2 decimal places)?
        ph = MaestroGUI.ph_value.get()
        newInfo = checkPhValue(ph)
        infoPair = (infoPair[0] and newInfo[0],infoPair[1] + newInfo[1])
        
        #check ASA cutoff percentage
        if(MaestroGUI.choosenASA.get()!= "none"):
            newInfo = checkRange(MaestroGUI.ASACutoff.get(),0,100,"ASA filter")
            infoPair = (infoPair[0] and newInfo[0],infoPair[1] + newInfo[1])
        
        #check display top mutations
        newInfo = checkRange(MaestroGUI.TopMutations.get(),1,1000,"Display top mutations")
        infoPair = (infoPair[0] and newInfo[0],infoPair[1] + newInfo[1])
        
    return infoPair
    
    
def checkResiduesTextbox(MaestroGUI, textbox, selectableResidues):
    """
    do emtries in text box fit to pattern?
    (are mutations valid?) check every single residue entry in text box
    @param MaestroGUI: gets whole GUI where data was inserted by user, so it can extract the values
    @param textbox: textbox of maestroGUI for fast/easy access
    @param selectableResidues: Mutate_from List (all residues)
    @return: infoPair: First Value says if validation is valid or not, Second Value says the errormessage if first is false
    @type infoPair: pair (boolean|str)
    """
    
    pattern = re.compile('^(([A-Z])[0-9]+\.[A-Z])\{([A-Z]([A-Z, ]*[A-Z]+)?)\}$')    #A1.A{T,L} A90.C{SMALL} ...
    
    for line in textbox.get('1.0', 'end-1c').splitlines():
            
            ##if MaestroGUI.choice.get() == 2 or 3
            if(len(line) < 7):
                errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, line("+line+") in text box has less than 7 characters!\n"
                print errorMsg
                return (False,errorMsg)
                #TODO do not abort, but do delete line
            #do Residues exist?
            if (pattern.match(line).group(1) not in selectableResidues):
                errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, residue in mutation("+line+") doesn't exist in molecule/protein!!\n"
                print errorMsg
                return (False,errorMsg)
            
            if (pattern.match(line) == None):
                errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, mutation("+line+") doesn't fit to pattern!\n Every line must look like e.g. A1.A{T,L} A90.C{SMALL} ..."
                print errorMsg
                return (False,errorMsg)
            
            #don't they Mutate to themselves?
            if (pattern.match(line).group(2) == pattern.match(line).group(3) and pattern.match(line).group(2) != 'X'):
                
                errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, mutation("+line+") mutates to it self!\n"
                print errorMsg
                return (False,errorMsg)
            
            if(MaestroGUI.choice.get() == 2):
                if(line == '[0]'):
                    errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, mutationlist is empty!\n Please add your required mutations in the 'List of Mutations'!\n"
                    print errorMsg
                    return (False,errorMsg)
                                    
            
    return (True,'')

#is ph between 5.5 and 8.5 (max. 2 decimal places)?
def checkPhValue(ph):
    """
    Is the pH value in the range of the servervalid ph-Value?
    @param ph: ph that should be checked 
    @return: param: True if in range otherwise false
    @type param: pair (boolean|str)
    """
    p = re.compile('^[5-8](\.[0-9]{1,2})?$')  #range will tested afterwards (NOT allowed: 7. or 07 or 7.000)
    if (p.match(ph) == None):
        errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, ph-value("+ph+") is not valid!\n Range: 5.50 to 8.50 (check decimal positions: max. 2)"
        print errorMsg
        return (False,errorMsg)
    elif (float(ph) < 5.5):
        errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, ph-value("+ph+") too low!\n Range: 5.50 to 8.50"
        print errorMsg
        return (False,errorMsg)
    elif(float(ph) > 8.5):
        errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, ph-value("+ph+") too high!\n Range: 5.50 to 8.50"
        print errorMsg
        return (False,errorMsg)
    
    return (True,'')
    
def checkRange(value, min_, max_, valuename):
    """
    check if tha given value is in the range between the given borders(min max)
    and makes the error Msg if too high/low
    
    @param value: given value of gui
    @param min_: lower border
    @param max_: upper border
    @param valuename: name of the given value
    @return: param: True if in range otherwise false
    @type param: pair (boolean|str)
     
    """
   
    if (float(value) < min_):
        errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, "+valuename+"("+value+") is not valid!\n Range: "+min_+" to "+max_
        print errorMsg
        return (False,errorMsg)
    elif(float(value) > max_):
        errorMsg = "\n ERROR\n MAESTRO Job wasn't sent, "+valuename+"("+value+") is not valid!\n Range: "+min_+" to "+max_
        print errorMsg
        return (False,errorMsg)
    
    return (True,'')


class AAMapping:
    '''
    In Chimera all residues are saved with the 3-letter-AminoAcid-Code but Server needs 1-letter-C so we need to translate them
    There wasn't any suitable function available, so we wrote a class with AAmapping functio
    We used 
    http://isites.harvard.edu/fs/docs/icb.topic1122119.files/dna2aa.py
    to create the class.
    '''
  
    def __init__(self):
        '''
        Create Map with all AA
        '''
        self.code = {'CYS': 'C', 'ASP': 'D', 'SER': 'S', 'GLN': 'Q', 'LYS': 'K',
                     'ILE': 'I', 'PRO': 'P', 'THR': 'T', 'PHE': 'F', 'ASN': 'N', 
                     'GLY': 'G', 'HIS': 'H', 'LEU': 'L', 'ARG': 'R', 'TRP': 'W', 
                     'ALA': 'A', 'VAL':'V', 'GLU': 'E', 'TYR': 'Y', 'MET': 'M'}
    
    def aaThreeToOne(self, AA):
        '''
        Translates one 3-letter-Code amino acid to 1-l-C
        if amino acid is not found in map: returns 'X'
        
        @param AA: should be a string with only one amino acid, case insensitive 
        '''
        AA.upper()
        result = ""
        
        if (AA in self.code):
            result = self.code[AA]
        else:
            result = "X"
        
        return result



class DataProviderMaestro:
    '''
    shall be used in apply, after validation
    collects data which shall be sent to BioDApps
    also needs job ID 
    '''
    
    
    def __init__(self):
        '''
        param will be added to data later
        Some params have fix values
        data is map which contains everything except of PDBs
        param is list of maps
        PDBs is List 
        later data and PDBs will be sent to BioDApps Server 
        '''
        self.data = {}
        self.param= [{}]
        self.PDBs = []
        self.data["service"]    = "Maestro"
        self.param[0]["bu"]        = False
        self.param[0]["model"]     = '-1'
        self.param[0]["browser"]   = False
    
    def readGui(self, MaestroGUI):
        '''
        extracts all needed data from GUI
        depending on choice of User (specific mutations or sensitivity profile) extraction differs (e.g. "evaluate" or "HotColdSpots")
        in specific mutations is another choice of User: single/multiple (don't get confused by this)
        @param MaestroGUI: gets GUI itself to extract data 
        '''
        
        #called in apply
        
        self.param[0]["chain"] = MaestroGUI.Chain_Menu.get()
        if (MaestroGUI.choice.get() == 2):
            self.param[0]["command"]        = "evaluate"    #or HotColdSpots
            self.param[0]["mutationlist"]   = MaestroGUI.text1.get(0.0, END)
            if(MaestroGUI.mutate_choice.get() == 1):
                self.param[0]["mutationtype"] = "single"
            else:   #if mutate_choice == 2
                self.param[0]["mutationtype"] = "multiple"
            self.param[0]["pH"] = MaestroGUI.ph_value.get()
            
        elif (MaestroGUI.choice.get() == 3):
            self.param[0]["command"]        = "scanMutations"
            #pdbid will be added later?
            
            experimentType="stabilize"
            if(MaestroGUI.experimentTye_choice.get() == 2):
                experimentType="destabilize"
            self.param[0]["experimentType"] = experimentType
            
            algorithm  = MaestroGUI.choosenAlgorithm.get()
            if (algorithm == "greedy search"):
                algorithm = "greedy"
            elif(algorithm == "evolutinary algorithm"):
                algorithm = "ea"
            else:   #SS-Scan
                algorithm = 'ssbond'
            self.param[0]["algorithmType"]  = algorithm
            self.param[0]["pH"]             = MaestroGUI.ph_value.get()
            self.param[0]["pointMutations"] = MaestroGUI.MaximumPoints.get()
            self.param[0]["asaFilter"]      = MaestroGUI.choosenASA.get()
            if(MaestroGUI.choosenASA.get() != "none"):
                self.param[0]["cutoff"]     = "0." + MaestroGUI.ASACutoff.get() #'cutoff': '0.3'
                if(MaestroGUI.ASACutoff.get()=="100"):
                    self.param[0]["cutoff"] = "1"
            self.param[0]["topMutants"] = MaestroGUI.TopMutations.get()
            
            #extract from textbox
            lines = self.terminateLineBreacks(MaestroGUI.text1)
            lines = lines.replace('}','},') #residues have to be separated by ',', there will also be a , at the last position in this string but it doesn't bother the server
            if (len(lines)>0):
                self.param[0]["mutationstring"] = lines
            
            # [{'pH': '7', 'cutoff': '0.3', 'CHAINID': '*', 'asaFilter': 'none', 'topMutants': '100', 'pointMutations': '3', 'command': 'scanMutations', 'browser': 'true', 'pdbId': '1bv1', 'algorithmType': 'greedy', 'MODELID': '-1', 'residuemask': '', 'experimentType': 'stabilize'}]
            # [{'pH': '7', 'cutoff': '0.3', 'CHAINID': '*', 'asaFilter': 'none', 'topMutants': '100', 'pointMutations': '3', 'command': 'scanMutations', 'browser': 'true', 'pdbId': '1bv1', 'algorithmType': 'greedy', 'MODELID': '-1', 'residuemask': '', 'experimentType': 'destabilize'}]
            # [{'pH': '7', 'cutoff': '0.3', 'asaFilter': 'none', 'topMutants': '1000000000', 'pointMutations': '', 'command': 'scanMutations', 'BIOUNIT': '1', 'pdbId': '1bv1', 'algorithmType': 'ssbond', 'browser': 'true', 'residuemask': '', 'experimentType': 'stabilize'}]
            # [{'pH': '7', 'cutoff': '0.3', 'CHAINID': '*', 'asaFilter': 'none', 'topMutants': '100', 'pointMutations': '3', 'command': 'scanMutations', 'browser': 'true', 'pdbId': '1bv1', 'algorithmType': 'ea', 'MODELID': '-1', 'residuemask': '', 'experimentType': 'stabilize'}]
            
        else:
            self.param[0]["command"]   = "HotColdSpots"    #or evaluate
        print MaestroGUI.Substructures
        self.param[0]["model"] = "f"#MaestroGUI.Substructures
    
    def terminateLineBreacks(self,textbox):
        """
        kills the linebraks of the textbox
        @param textbox: textbox of maestroGUI
        @return: text: textbox text without linebraks
        @type text: str
        """
        text=""
        for line in textbox.get('1.0', 'end-1c').splitlines():
            if(len(line) > 1):
                text = text + line.replace("\n",",")
        return text
        
        
    def setFilename(self,filename):
        '''
        saves filename in data construct
        @param filename: gets name of pdb-file (can be any name: has nothing to do with Chimera)
        '''
        self.param[0]["pdbFile"]=filename
    
    #returns -1 if pdbFile not defined 
    def getFilename(self):
        '''
        return the filename if not None (otherwise -1
        @returns -1 if pdbFile not defined or pdbFile which shall be sent to BioDApps Server
        @type int or str
        '''
        if(self.param[0].has_key("pdbFile")):
            return self.param[0]["pdbFile"]
        else:
            return -1
    
    def setJobId(self,JobId):
        '''
        saves JobId in construct which will be sent to BioDApps
        @param JobId: retrieved from Server 
        '''
        self.data["jobId"]=JobId
    
    #returns -1 if JobId not defined 
    def getJobId(self):
        '''
        if JobId has already been set it will be returned
        otherwise -1 will be returned
        @returns -1 if jobID not defined or pdbFile which shall be sent to BioDApps Server
        @type int or str
        '''
        if(self.data.has_key("jobId")):
            return self.data["jobId"]
        else:
            return -1
        
    def getJson(self):
        '''
        returns whole data construct which will be send to BioDApps Server
        Won't be changed anymore before sanding
        has to put the list 'param' into 'data' (data will be returned)
        @return self.data: returns json
        @type str
        '''
        self.data["param"] = self.param
        return self.data
    
    def setPdbs(self, pdbList):
        '''
        pdbList shall only contain one single pdb (because it's MAESTRO in PIRATES we need more)
        @param pdbList: List with all PDBs which shall be sent to Server, in this case only one PDB!
        ''' 
        for pdb in pdbList:
            self.PDBs.append(pdb)
    
    def getPdbs(self):
        '''
        only returns tuple (without list)!
        '''
        return self.PDBs[0] #only returns tuple (without list)


    